
function makeReplaceTable ()
	if not (global) then global = {} end
	if (global.replaceTable) then return end
	global.replaceTable = {
		{name = "transport-belt", type = "transport-belt" , tier = 10},
		{name = "underground-belt", type = "underground-belt" , tier = 10},
		--{name = "loader", type = "loader" , tier = 10},
		--{name = "splitter", type = "splitter" , tier = 10},

		{name = "fast-transport-belt", type = "transport-belt" , tier = 20},
		{name = "fast-underground-belt", type = "underground-belt" , tier = 20},
		--{name = "fast-loader", type = "loader" , tier = 20},
		--{name = "fast-splitter", type = "splitter" , tier = 20},

		{name = "express-transport-belt", type = "transport-belt" , tier = 30},
		{name = "express-underground-belt", type = "underground-belt" , tier = 30}
		--{name = "express-loader", type = "loader" , tier = 30},
		--{name = "express-splitter", type = "splitter" , tier = 30}
		}
end


function printAll(text)
	if type(text) == "table" then text = serpent.block(text, {comment = false}) end
	for player_index, player in pairs (game.players) do game.players[player_index].print (text) end
end

-- on init

script.on_init(function ()
	global  = global or {}
	global.enabled = global.enabled or true
	for player_index, player in pairs (game.players) do
		add_GUI (player_index)
	end
end)

-- on user created

script.on_event(defines.events.on_player_created, function (event)
	global  = global or {}
	global.enabled = global.enabled or true
	local player_index = event.player_index or event.player.index
	add_GUI (player_index)
end)


function add_GUI_frame (player, GUI_frame_name)
	player.gui.left.add{type = "frame", name = GUI_frame_name, direction = "horizontal"}
	return player.gui.left.AU_frame
end

function add_GUI_button (player, parent, GUI_button_name)
	parent.add {type = "button", name = GUI_button_name, caption = GUI_button_name}
end

-- create gui

function add_GUI (player_index)
	local player = game.players[player_index]
	local parant_frame = add_GUI_frame (player, "AU_frame")
	add_GUI_button (player, parant_frame, "AU_button")
	return 
end


function chunk_iterator(surface)
	local surface = surface or game.surfaces[1]
	local surface_table = {}
	for chunk in surface.get_chunks() do
		--game.player.print("x: " .. chunk.x .. ", y: " .. chunk.y)
		surface_table[#surface_table+1] = {index = (#surface_table+1), pos = {x = chunk.x, y = chunk.y}}
	end
	return surface_table
end


function find_lines (chunk, surface)
	local chunk_x = chunk.pos.x * 32
	local chunk_y = chunk.pos.y * 32
	local new_lines = {}
	local limit = 10
	
	for x = (chunk_x), ((chunk_x)+32) do
		local area = {left_top  = {x, chunk_y}, right_bottom  = {x+1, chunk_y+32}}
		local n_ents = surface.count_entities_filtered{area = area, type="transport-belt"}
		if n_ents > limit then
			--printAll("horizontal line " .. n_ents)
			local new_line = {area = area, length = n_ents, horizontal = false}
			new_lines[#new_lines+1] = new_line
		end
	end
	
	for y = (chunk_y), ((chunk_y)+32) do
		local area = {left_top  = {chunk_x, y}, right_bottom  = {chunk_x+32, y+1}}
		local n_ents = surface.count_entities_filtered{area = area, type="transport-belt"}
		if n_ents > limit then
			--printAll("vertical line " .. n_ents)
			local new_line = {area = area, length = n_ents, horizontal = true}
			new_lines[#new_lines+1] = new_line
		end
	end
	--printAll("new_lines " .. #new_lines)
	return new_lines
end


function destroy_lines (lines_list, surface)
	local force = game.players[1].force -- not for multiplayer!
	for i, line in pairs (lines_list) do
		surface.deconstruct_area{area=line.area, force = force}
	end
end


function vertical_belts_finder (line, surface)
	--area = {left_top  = {x, chunk_y}, right_bottom  = {x+1, chunk_y+32}}
	local x = line.area.left_top[1]+0.5
	local first_y = line.area.left_top[2]+0.5
	local  last_y = line.area.right_bottom[2]-0.5
	--printAll("x="..x .. " first_y="..first_y .. " last_y="..last_y)
	local last_was_ok = false
	local belts_lines = {}
	local last_direction = nil
	for y = first_y, last_y do
		local belt_entity = surface.find_entity("express-transport-belt", {x=x, y=y})

		if ((belt_entity) and not (last_was_ok)) then
			local direction = belt_entity.direction
			--printAll("Entity was first " .. belt_entity.name)
			--belts_lines[#belts_lines+1] = {first_entity = belt_entity, first_pos = {x=x, y=y}, last_entity = nil, last_pos = {}, direction = direction, legit = false}
			belts_lines[#belts_lines+1] = {first_pos = {x=x, y=y}, last_pos = {}, direction = direction, legit = false}
			last_was_ok = true
			last_direction = direction
		end
		
		if ((belt_entity) and (last_was_ok)) then
			local direction = belt_entity.direction
			if (direction==last_direction) then
				--printAll("Entity was middle ")
				--belts_lines[#belts_lines].last_entity = belt_entity
				belts_lines[#belts_lines].last_pos = {x=x, y=y}
				local length = (y-1) - belts_lines[#belts_lines].first_pos.y
				belts_lines[#belts_lines].length = length
				belts_lines[#belts_lines].legit = true
				last_was_ok = true
			else
				--printAll("direction: " .. direction .. " last_direction: " .. last_direction)
				last_was_ok = false
			end
		end
		
		if (not (belt_entity) and (last_was_ok)) then
			last_was_ok = false
			last_direction = nil
		end
	end
	if belt_lines then
		printAll("08v: belt_lines:" .. #belt_lines)
		else
		printAll("08v: nothing is founded")
	end
	return belts_lines
end


function horizontal_belts_finder (line, surface)
	local y = line.area.left_top[2]+0.5
	local first_x = line.area.left_top[1]+0.5
	local  last_x = line.area.right_bottom[1]-0.5
	local last_was_ok = false
	local belts_lines = {}
	local last_direction = nil
	for x = first_x, last_x do
		local belt_entity = surface.find_entity("express-transport-belt", {x=x, y=y})

		if ((belt_entity) and not (last_was_ok)) then
			local direction = belt_entity.direction
			--belts_lines[#belts_lines+1] = {first_entity = belt_entity, first_pos = {x=x, y=y}, last_entity = nil, last_pos = {}, direction = direction, legit = false}
			belts_lines[#belts_lines+1] = {first_pos = {x=x, y=y}, last_pos = {}, direction = direction, legit = false}
			last_was_ok = true
			last_direction = direction
		end
		
		if ((belt_entity) and (last_was_ok)) then
			local direction = belt_entity.direction
			if (direction==last_direction) then
				--belts_lines[#belts_lines].last_entity = belt_entity
				belts_lines[#belts_lines].last_pos = {x=x, y=y}
				local length = (x-1) - belts_lines[#belts_lines].first_pos.x
				belts_lines[#belts_lines].length = length
				belts_lines[#belts_lines].legit = true
				last_was_ok = true
			else
				last_was_ok = false
			end
		end
		
		if (not (belt_entity) and (last_was_ok)) then
			last_was_ok = false
			last_direction = nil
		end
	end
	if belt_lines then
		printAll("08h: belt_lines:" .. #belt_lines)
		else
		printAll("08h: nothing is founded")
	end
	return belts_lines
end


function find_belts_in_lines (lines_list, surface)
	--printAll("find_belts_in_lines")
	local belt_lines = {}
	for i, line in pairs (lines_list) do
		if (line.horizontal) then
			local new_belt_lines = horizontal_belts_finder (line, surface)
			for i, v in pairs (new_belt_lines) do
				table.insert(belt_lines, v)
			end
			printAll("10h: new_lines "..#new_belt_lines)
		else
			local new_belt_lines = vertical_belts_finder (line, surface)
			for i, v in pairs (new_belt_lines) do
				table.insert(belt_lines, v)
			end
			printAll("10v: new_lines "..#new_belt_lines)
		end
	end
	printAll("16: belt_lines:" .. #belt_lines)
	return belt_lines
end


function find_chunk_lines (chunks, surface)
	local belt_lines = {}
	-- lines[i] = {from = {pos={x=x, y=y}}, to = {pos={x=x, y=y}}, name = name}
	for i, chunk in pairs (chunks) do
		--printAll("i chunks = " .. i .. " - " .. serpent.block(chunk, {comment = false}))
		local new_lines = find_lines (chunk, surface)
		--destroy_lines (new_lines, surface)
		if new_lines and (#new_lines>0) then
			local new_belt_lines = find_belts_in_lines (new_lines, surface)
			for i, v in pairs (new_belt_lines) do
				table.insert(belt_lines, v)
			end
		end
	end
	printAll("32: belt_lines:" .. #belt_lines)
	
	return belt_lines
end


function filter_chunks (chunks, surface)
	-- optimization, we don't need to iterate all chunks, just with belts!
	local new_chunks = {}

	for i, chunk in pairs (chunks) do
		local chunk_x = chunk.pos.x * 32
		local chunk_y = chunk.pos.y * 32
		local limit = 5 -- max amound of counting
		local n_entities = surface.count_entities_filtered{area = {left_top  = {chunk_x, chunk_y}, right_bottom  = {chunk_x+32, chunk_y+32}}, type="transport-belt", limit = limit}
		if n_entities == limit then
			new_chunks[#new_chunks+1] = chunk
		end
	end
	printAll("filter_chunks: chunks was: " .. #chunks .. ", chunks now: " .. #new_chunks)
	chunks = new_chunks
end


function replacer (surface, position, direction, underground_belt)
	local force = game.players[1].force
	old_entity = surface.find_entity("express-transport-belt", position)
	if old_entity then
		old_entity.destroy()
	end
	
	--- problem
	--surface.create_entity{name="express-underground-belt", position=position, direction=direction, force=force,fast_replace=false, ["underground-belt"]= underground_belt}
	surface.create_entity{name="express-underground-belt", position=position, direction=direction, force=force,fast_replace=false, type=underground_belt}
	
	printAll("Replaced " .. serpent.block(position, {comment = false}))
end

deltete 

--function delete_all_between (surface, belt_line)

--end

function replace_belts_with_underground_belt (surface, belt_lines)
	printAll("74: belt_lines = " .. serpent.block(belt_lines, {comment = false}))
	for i, belt_line in pairs (belt_lines) do
		--printAll(i .. " belt_line = " .. serpent.block(belt_line, {comment = false}))

		local first_pos = belt_line.first_pos
		local last_pos = belt_line.last_pos
		
		local direction = belt_line.direction
		if direction == 0 then -- to the up -y
			printAll("direction: " .. direction)
			replacer (surface, first_pos, direction, "output")
			replacer (surface, last_pos, direction, "input")
		end
		if direction == 2 then -- to the right +x
			printAll("direction: " .. direction)
			replacer (surface, first_pos, direction, "input")
			replacer (surface, last_pos, direction, "output")
		end
		if direction == 4 then -- to the down +y
			printAll("direction: " .. direction)
			replacer (surface, first_pos, direction, "input")
			replacer (surface, last_pos, direction, "output")
		end
		if direction == 6 then -- to the up -x
			printAll("direction: " .. direction)
			replacer (surface, first_pos, direction, "output")
			replacer (surface, last_pos, direction, "input")
		end
	end
	
end




function on_click_main(surface)	
	local chunks = chunk_iterator()
	local surface = surface or game.surfaces[1]
	--printAll("#chunks = " .. #chunks)
	
	filter_chunks (chunks, surface)
	local belt_lines = find_chunk_lines (chunks, surface)
	printAll("64: belt_lines:" .. #belt_lines)
	printAll("64: belt_lines = " .. serpent.block(belt_lines, {comment = false}))
	replace_belts_with_underground_belt (surface, belt_lines)
end

-- on_gui_click

script.on_event(defines.events.on_gui_click, function (event)
	if event.element.name == "AU_button" then
		global.enabled = global.enabled or false
		global.enabled = (not (global.enabled))
		on_click_main()
	end
end)















































